if (!require(stringr)) {
  install.packages("stringr")
  library(stringr)  
}
if (!require(magick)) {
  install.packages("magick")
  library(magick)
}

if (!require(pdftools)) {
  install.packages("pdftools")
  library(pdftools)
}


func_extraer_fotos = function(ficheropdf = "Ficha.pdf",
                              raiz_nombrefotos = "foto") {
  
  # Paso 1:    
  comando = paste0("./pdfimages -list ",ficheropdf," ",raiz_nombrefotos," > salida.txt")
  # En Windows, descomentar la siguiente orden
  #comando = paste0("pdfimages.exe -list ",ficheropdf," ",
  #                 raiz_nombrefotos," > salida.txt")
  system(comando)
  
  # Paso 2:
  salida = readLines("salida.txt")
  # escudo en líneas con las siguientes características
  # foto-0000.ppm: page=1 width=150 height=150 hdpi=120.00 vdpi=120.00 colorspace=DeviceRGB bpc=8
  donde_escudo = str_detect(salida,"width=150")
  cuales_escudo = which(donde_escudo)
  nombres = str_split(salida,": ")
  vnombres = c()
  for (i in 1:length(salida)) {
    if ((i %in% cuales_escudo)) {
      vvnombre = nombres[[i]][1]
      unlink(vvnombre)
    } else {
      vnombres = c(vnombres,nombres[[i]][1])
    }
  }
  
  # Paso 2b: detecta fotos faltantes: N
  ffal = c("E")
  for (i in 2:length(donde_escudo)) {
    img_ant = donde_escudo[i-1]
    img_act = donde_escudo[i]
    if (img_act) { # actual escudo: E
      if (img_ant) { # anterior escudo: E
        ffal = c(ffal,"N","E") # dos escudos seguidos: falta foto
      } else { # anterior foto:
        ffal = c(ffal,"E") # foto y escudo: escudo
      }
    } else { # actual foto: F
      if (img_ant) { # anterior escudo: E
        ffal = c(ffal,"F") # escudo y foto: sí hay foto
      } else { # anterior foto:
        ffal = c(ffal,"E","F") # foto y foto: falta escudo
      }
    }
  }
  ifotos = which(!(ffal=="E"))
  qfotos = ffal[ifotos]
  sifotos = which(qfotos=="F")
  
  if (!dir.exists("fotos")) {
    dir.create("fotos")
  }
  
  # Paso 3:
  cadenafmt = paste0("fotos/",raiz_nombrefotos,"%02d.png") # "fotos/foto%02d.png"
  #vsnombres = sprintf(cadenafmt,1:length(vnombres))
  vsnombres = sprintf(cadenafmt,sifotos)
  for (i in 1:length(vnombres)) {
    fichero = image_read(vnombres[i])
    fichero_png = image_convert(fichero,"png")
    image_info(fichero_png)
    image_write(fichero_png,path = vsnombres[i])
    unlink(vnombres[i])
  }
  
}



############################
#### extraer fotos y nombre
############################


func_extraer_fotos_mej = function(ficheropdf = "Ficha.pdf",
                              raiz_nombrefotos = "foto") {
  
  # Paso 1:    
  comando = paste0("./pdfimages -list ",ficheropdf," ",raiz_nombrefotos," > salida.txt")
  # En Windows, descomentar la siguiente orden
  #comando = paste0("pdfimages.exe -list ",ficheropdf," ",
  #                 raiz_nombrefotos," > salida.txt")
  system(comando)
  
  # Paso 2:
  salida = readLines("salida.txt")
  # escudo en líneas con las siguientes características
  # foto-0000.ppm: page=1 width=150 height=150 hdpi=120.00 vdpi=120.00 colorspace=DeviceRGB bpc=8
  donde_escudo = str_detect(salida,"width=150")
  cuales_escudo = which(donde_escudo)
  nombres = str_split(salida,": ")
  vnombres = c()
  for (i in 1:length(salida)) {
    if ((i %in% cuales_escudo)) {
      vvnombre = nombres[[i]][1]
      unlink(vvnombre)
    } else {
      vnombres = c(vnombres,nombres[[i]][1])
    }
  }
  
  # Paso 2b: detecta fotos faltantes: N
  ffal = c("E")
  for (i in 2:length(donde_escudo)) {
    img_ant = donde_escudo[i-1]
    img_act = donde_escudo[i]
    if (img_act) { # actual escudo: E
      if (img_ant) { # anterior escudo: E
        ffal = c(ffal,"N","E") # dos escudos seguidos: falta foto
      } else { # anterior foto:
        ffal = c(ffal,"E") # foto y escudo: escudo
      }
    } else { # actual foto: F
      if (img_ant) { # anterior escudo: E
        ffal = c(ffal,"F") # escudo y foto: sí hay foto
      } else { # anterior foto:
        ffal = c(ffal,"E","F") # foto y foto: falta escudo
      }
    }
  }
  ifotos = which(!(ffal=="E"))
  qfotos = ffal[ifotos]
  sifotos = which(qfotos=="F")
  
  # Paso 2c: extrae apellidos y nombre de los alumnos
  text <- pdf_text(ficheropdf)
  
  dat01 = c() # contiene apellidos y nombre
  dat02 = c() # contiene apellidos y nombre separados por guiones
  for (j in 1:length(text)) {
    pagina = str_split(text[j],"\n")
    lineas = unlist(pagina)
    for (i in 1:length(lineas)) {
      res = str_detect(lineas[i],"Apellidos, nombre")
      if (res) {
        ss = unlist(str_split(lineas[i],"Apellidos, nombre"))
        ss1 = str_trim(ss[2])
        dat01 = c(dat01,ss1)
        #dat02 = c(dat02,str_remove_all(str_remove_all(ss1,""),","))
        dat02 = c(dat02,str_remove_all(str_replace_all(ss1," ","_"),","))
      }
    }
  }
  nb_sifotos = dat02[sifotos]
  
  
  if (!dir.exists("fotos")) {
    dir.create("fotos")
  }
  
  # Paso 3:
  cadenafmt = paste0("fotos/",raiz_nombrefotos,"%02d_%s.png") # "fotos/foto%02d.png"
  vsnombres = sprintf(cadenafmt,sifotos,nb_sifotos)
  for (i in 1:length(vnombres)) {
    fichero = image_read(vnombres[i])
    fichero_png = image_convert(fichero,"png")
    image_info(fichero_png)
    image_write(fichero_png,path = vsnombres[i])
    unlink(vnombres[i])
  }
  
}


############################
############################



